import boto3
from datetime import datetime, timezone

from .caching_logic import get_decorator, get_value
from .exceptions import InvalidS3UriError


def cache(
    s3Uri: str, max_age_in_seconds=60, entry_name=False, check_before_download=True
):
    """
    Calls parameter caching, and decorates function by injecting key value into the context object

    Args:
        s3Uri(string): S3 Uri of file to download
        max_age_in_seconds(int) : Time to Live of the parameter in seconds
        entry_name(str) : Name of entry in cache
        check_before_download (boolean): Check object age before downloading
    Returns:
        decorate         : Decorated function

    """

    decorator = get_decorator(
        argument=s3Uri,
        max_age_in_seconds=max_age_in_seconds,
        entry_name=entry_name,
        miss_function=get_object_from_s3,
        check_before_download=check_before_download,
        send_details=True,  # specifies to send all details to miss_function
        s3Uri=s3Uri,
    )
    return decorator


def get_entry(
    s3Uri: str, max_age_in_seconds=60, entry_name=False, check_before_download=True
):
    """
    Wrapper function for parameter_caching

    Args:
        parameter(string): Name of the parameter in System Manager Parameter Store
        max_age_in_seconds(int) : Time to Live of the parameter in seconds
        var_name(string) : Optional name of parameter to inject into context object

    Returns:
        parameter_value(string)  : Value of the parameter
    """

    file_location = get_value(
        argument=s3Uri,
        max_age_in_seconds=max_age_in_seconds,
        entry_name=entry_name,
        miss_function=get_object_from_s3,
        check_before_download=check_before_download,
        send_details=True,  # specifies to send all details to miss_function
        s3Uri=s3Uri,
    )

    return file_location


def get_object_from_s3(**kwargs):

    """
    Gets parameter value from the System manager Parameter store

    Args:
        kwargs['s3Uri']: Uri of S3 object in the form of s3://bucket-name/path/to/object
        kwargs['entry_name']: Name of entry in cache
        kwargs['entry_age_in_seconds']: Age of cache entry
    Returns:
        file_location (str): location of file on file-system (e.g. /tmp/example.txt)

    """
    s3_uri = kwargs["s3Uri"]
    bucket_name, key = parse_s3_uri(s3_uri)
    file_location = f"/tmp/{kwargs['entry_name']}"

    s3 = boto3.resource("s3")
    s3_object = s3.Object(bucket_name, key)

    if kwargs["check_before_download"] and kwargs["entry_age_in_seconds"] is not None:
        last_modified = s3_object.last_modified
        now = datetime.now(timezone.utc)
        object_age_in_seconds = (now - last_modified).seconds

        # if the object is older than the entry_age
        if object_age_in_seconds > kwargs["entry_age_in_seconds"]:
            return file_location

    s3_object.download_file(file_location)

    return file_location


def parse_s3_uri(s3_uri):

    elements = s3_uri.split("/")

    if elements[0] != "s3:" or elements[1] != "":
        raise InvalidS3UriError(s3_uri)
    else:
        bucket_name = elements[2]
        key = "/".join(elements[3:])

    return bucket_name, key



class LambdaCacheError(Exception):

    """
    Base class for exceptions in this module.
    """

    pass


class ArgumentTypeNotSupportedError(LambdaCacheError):

    """
    Raised when Argument is not supported by the function.
    """

    def __init__(self, message):
        self.message = message
        self.Code = "ArgumentTypeNotSupportedError"


class NoEntryNameError(LambdaCacheError):

    """
    Raised when No entry_name is provided.
    """

    def __init__(self, message=False):
        self.message = "No entry_name provided"
        self.Code = "NoEntryNameError"


class InvalidS3UriError(LambdaCacheError):

    """
    s3Uri provided in invalid format
    """

    def __init__(self, invalid_uri):
        self.message = f"Expected Valid s3uri of the form 's3://bucket-name/path/to/file', given: {invalid_uri}"
        self.Code = "InvalidS3UriError"



import boto3

from .caching_logic import get_decorator, get_value
from .exceptions import ArgumentTypeNotSupportedError

default_max_age_in_seconds = 60


def cache(name, max_age_in_seconds=default_max_age_in_seconds, entry_name=False):
    """
    Calls check cache, and decorates function by injecting key value into the context object
        ** The secret name must be ASCII letters, digits, or the following characters : /_+=.@-
    Args:
        name(string)     : Name of the secret (or ARN)
        ttl_seconds(int) : Time to Live of the parameter in seconds
        var_name(string) : Optional name of parameter to inject into context object

    Returns:
        decorate         : Decorated function

    """

    decorator = get_decorator(
        argument=name,
        max_age_in_seconds=max_age_in_seconds,
        entry_name=entry_name,
        miss_function=get_secret_from_secrets_manager,
    )
    return decorator


def get_entry(name, max_age_in_seconds=default_max_age_in_seconds, entry_name=False):
    """
    Wrapper function for parameter_caching

    Args:
        name(string): Name of the Secret in Secrets Manager (arn is also accepted)
        ttl_seconds(int) : Time to Live of the parameter in seconds
        var_name(string) : Optional name of parameter to inject into context object

    Returns:
        secret_value(string)  : Value of the parameter
    """
    secret_value = get_value(
        argument=name,
        max_age_in_seconds=max_age_in_seconds,
        entry_name=entry_name,
        miss_function=get_secret_from_secrets_manager,
    )
    return secret_value


def get_secret_from_secrets_manager(name):
    """
    Gets parameter value from the System manager Parameter store

    Args:
        name(string): Name of the secret or ARN of secret
    Returns:
        secret_value (string/binary): Value of secret in secrets manager in either String or Binary format
    """

    if isinstance(name, str):
        secrets_client = boto3.client("secretsmanager")
        response = secrets_client.get_secret_value(SecretId=name)
        if response.get("SecretString") is not None:
            return_value = response["SecretString"]
        else:
            return_value = response["SecretBinary"]
    else:
        raise ArgumentTypeNotSupportedError(
            f"Secrets Manager only supports str arguments: {name} is not a string"
        )

    return return_value



# -*- coding: utf-8 -*-

"""
lambda-cache
~~~~~~~~~~~~

A python package for caching within AWS Lambda Functions

Full Documentation is at <https://lambda-cache.rtfd.io>.
:license: MIT, see LICENSE for more details.
"""

__version__ = "0.8.1"

from .ssm import cache, get_entry
from .secrets_manager import cache, get_entry
from .s3 import cache, get_entry



import boto3

from .caching_logic import get_decorator, get_value, get_entry_name
from .exceptions import ArgumentTypeNotSupportedError

default_max_age_in_seconds = 60


def cache(
    parameter: str, max_age_in_seconds=default_max_age_in_seconds, entry_name=False
):
    """
    Calls parameter caching, and decorates function by injecting key value into the context object

    Args:
        parameter(string): Name of the parameter in System Manager Parameter Store
        max_age_in_seconds(int) : Time to Live of the parameter in seconds
        var_name(string) : Optional name of parameter to inject into context object

    Returns:
        decorate         : Decorated function

    """

    decorator = get_decorator(
        argument=parameter,
        max_age_in_seconds=max_age_in_seconds,
        entry_name=entry_name,
        miss_function=get_parameter_from_ssm,
    )
    return decorator


def get_entry(
    parameter: str, max_age_in_seconds=default_max_age_in_seconds, entry_name=False
):
    """
    Wrapper function for parameter_caching

    Args:
        parameter(string): Name of the parameter in System Manager Parameter Store
        max_age_in_seconds(int) : Time to Live of the parameter in seconds
        var_name(string) : Optional name of parameter to inject into context object

    Returns:
        parameter_value(string)  : Value of the parameter
    """

    parameter_value = get_value(
        argument=parameter,
        max_age_in_seconds=max_age_in_seconds,
        entry_name=entry_name,
        miss_function=get_parameter_from_ssm,
    )
    return parameter_value


def get_parameter_from_ssm(parameter):
    """
    Gets parameter value from the System manager Parameter store

    Args:
        parameter(string / list): Name of the parameter(s) in System Manager Parameter Store
    Returns:
        parameter_value (string): Single Value of parameter in Parameter Store; or
        parameters (list): List of Values from Parameter Store
    """

    ssm_client = boto3.client("ssm")

    if isinstance(parameter, str):
        response = ssm_client.get_parameter(Name=parameter, WithDecryption=True)
        parameter_value = response["Parameter"]["Value"]
        # return StringList
        if response["Parameter"]["Type"] == "StringList":
            parameter_value = parameter_value.split(",")
        return_value = parameter_value

    elif isinstance(parameter, list):
        response = ssm_client.get_parameters(Names=parameter, WithDecryption=True)
        parameters = {}
        for param in response["Parameters"]:
            param_name = get_entry_name(param["Name"], False)
            if param["Type"] == "StringList":
                parameters[param_name] = param["Value"].split(",")
            else:
                parameters[param_name] = param["Value"]
        return_value = parameters

    else:
        raise ArgumentTypeNotSupportedError(
            "Only str or list supported for ssm, {parameter} is of type: {type(parameter)}"
        )

    return return_value



import time
import functools

from .exceptions import ArgumentTypeNotSupportedError, NoEntryNameError


def get_decorator(**kwargs):

    """
    Args:
        argument (string, list, dict) : argument to be passed to the missed function
        max_age_in_seconds(int) : Time to Live of the entry in seconds
        entry_name(string) : Name of entry in cache, is also the name of the entry in the event object
        miss_function(function): Function to execute when there is a miss on the cache or cache is expired
        - Any additional kwargs to be passed to miss_function.
    return:
        Decorator of the function
    """

    def decorator(func):
        @functools.wraps(func)
        def inner_function(event, context):

            response = check_cache(**kwargs)

            # Inject {parameter_name: parameter_value} into context object
            for key in response:
                setattr(context, key, response[key])

            return func(event, context)

        return inner_function

    return decorator


def get_value(**kwargs):

    """
    returns value of check_cache.
    """
    response = check_cache(**kwargs)
    parameter_value = list(response.values())[0]
    return parameter_value


def check_cache(
    argument,
    max_age_in_seconds,
    entry_name,
    miss_function,
    send_details=False,
    **kwargs
):

    """
    Executes the caching logic, checks cache for entry
    If entry doesn't exist, returns entry_value by calling the miss function with entry_name and var_name
    If entry does exist check entry_age:
        If entry_age < max_age_in_seconds, returns value from cache
        If entry_age >= max_age_in_seconds, returns value by calling miss_function

    Args:
        argument (string, list, dict) : argument to be passed to the missed function
        max_age_in_seconds(int) : Time to Live of the entry in seconds
        entry_name(string) : Name of entry in cache, is also the name of the entry in the event object
        miss_function(function): Function to execute when there is a miss on the cache or cache is expired
    Returns:
        entry_value(dict)  : {entry_name: entry_value}
    """

    entry_name = get_entry_name(argument, entry_name)
    entry_age_in_seconds = get_entry_age(entry_name)

    # if kwargs exist, then pass additional data to miss_function, else just argument
    if send_details:
        kwargs["argument"] = argument
        kwargs["entry_name"] = entry_name
        kwargs["entry_age_in_seconds"] = entry_age_in_seconds
        kwargs["max_age_in_seconds"] = max_age_in_seconds

    if entry_age_in_seconds is None:
        if send_details:
            entry_value = miss_function(**kwargs)
        else:
            entry_value = miss_function(argument)
        update_cache(entry_name, entry_value)

    elif entry_age_in_seconds < max_age_in_seconds:
        entry_value = get_entry_from_cache(entry_name)

    else:
        if send_details:
            entry_value = miss_function(**kwargs)
        else:
            entry_value = miss_function(argument)
        update_cache(entry_name, entry_value)

    return {entry_name: entry_value}


def get_entry_name(argument, entry_name):

    """
    argument is either SSM Parameter, Secret in Secrets Manager or Key in S3 bucket:
        SSM Parameter names can include only the following symbols and letters: a-zA-Z0-9_.-/
        Secret name must be ASCII letters, digits, or the following characters : /_+=.@-
        S3 Keys can have a varied characters

    if entry_name is set, we return entry_name
    if entry_name is False,
        if entry_name is a string, Default entry_name to the string after the last '/' in argument

    Args:
        argument (string, list, dict) : argument to be passed to the missed function
        entry_name(string) : Optional name of entry in cache, and variable injected into event object
    Returns:
        cache_entry_name   : Name of Entry in the cache
    """

    if isinstance(argument, str):
        if entry_name:
            cache_entry_name = entry_name
        else:
            cache_entry_name = argument.split("/")[-1]

    elif type(argument) in [int, list, dict]:
        if not entry_name:
            raise NoEntryNameError(
                "You must specify an entry_name for arguments of type list, dict or int"
            )
        else:
            cache_entry_name = entry_name
    else:
        raise ArgumentTypeNotSupportedError(
            "Argument can only be of Type str, int, list or dict"
        )

    return cache_entry_name


def get_entry_age(entry_name):

    """
    Args:
        entry_name(string): Name of entry to get age for
    Returns:
        entry_age_seconds(int): Age of entry in seconds, returns None if no entry exist
    """
    global global_aws_lambda_cache

    try:
        get_param_timestamp = global_aws_lambda_cache[entry_name][
            "last_updated_timestamp"
        ]
        entry_age = int(time.time() - get_param_timestamp)

    # cache doesn't exist. Create it.
    except NameError:
        global_aws_lambda_cache = {
            entry_name: {"value": None, "last_updated_timestamp": None}
        }
        entry_age = None

    # entry doesn't exist in cache or is still None (due to partial failure)
    except (KeyError, TypeError):
        global_aws_lambda_cache[entry_name] = {
            "value": None,
            "last_updated_timestamp": None,
        }
        entry_age = None

    return entry_age


def update_cache(entry_name, entry_value):
    global global_aws_lambda_cache

    global_aws_lambda_cache[entry_name] = {
        "value": entry_value,
        "last_updated_timestamp": time.time(),
    }

    return


def get_entry_from_cache(entry_name):

    """
    Gets entry value from the cache

    Args:
        entry_name (string): Name of the entry in cache
    Returns:
        entry_value   (any): Value of entry in cache
    """

    global global_aws_lambda_cache
    entry_value = global_aws_lambda_cache.get(entry_name).get("value")
    return entry_value



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['lambda_cache']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'lambda-cache',
    'version': '0.8.1',
    'description': 'Python utility for simple caching in AWS Lambda functions',
    'long_description': '<h1 align="center"> Lambda Cache </h1>\n<h2 align="center"> Simple Caching for AWS Lambda</h2>\n\n![PackageStatus](https://img.shields.io/static/v1?label=status&message=beta&color=orange?style=flat-square) ![PyPI version](https://img.shields.io/pypi/v/lambda-cache) ![PythonSupport](https://img.shields.io/static/v1?label=python&message=3.6%20|%203.7%20|%203.8&color=blue?style=flat-square&logo=python) ![License: MIT](https://img.shields.io/github/license/keithrozario/lambda-cache) [![Documentation Status](https://readthedocs.org/projects/lambda-cache/badge/?version=latest)](https://lambda-cache.readthedocs.io/en/latest/?badge=latest) [![Documentation Status](https://readthedocs.org/projects/lambda-cache/badge/?version=latest)](https://lambda-cache.readthedocs.io/en/latest/?badge=latest)\n\n![Test](https://github.com/keithrozario/lambda-cache/workflows/Test/badge.svg?branch=release) [![Coverage Status](https://coveralls.io/repos/github/keithrozario/lambda-cache/badge.svg?branch=release)](https://coveralls.io/github/keithrozario/lambda-cache?branch=release) [![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black) \n\n# Introduction\n\n<p align="center"><img src=\'docs/images/lambda_cache.png\'></p>\n\n_lambda-cache_ helps you cache data in your Lambda function **across** invocations. The package utilizes the internal memory of the lambda function\'s [execution context](https://docs.aws.amazon.com/lambda/latest/dg/runtimes-context.html) to store data across multiple invocations. Doing this:\n\n* Reduces load on back-end systems\n* Reduces the execution time of the lambda\n* Guarantees functions will reference latest data after cache expiry\n* Reduces calls to APIs throttling limits or high charges\n\n_lambda-cache_ prioritizes simplicity over performance and flexibility. \n\n\nThe package is purpose-built for AWS Lambda functions, and currently supports SSM Parameters, Secrets from Secrets Manager and S3 Objects.\n\n\n# Installation\n\nInclude the package in your functions code zip-file using the following:\n\n    $ pip install lambda-cache -t /path/of/function\n\nRefer to [docs](https://lambda-cache.readthedocs.io/en/latest/install/) for how to include into your lambda function.\n\n# Usage\n\nBelow we describe further features of the package. For more info refer to the [user guide](https://lambda-cache.readthedocs.io/en/latest/user_guide/).\n\n<!-- vscode-markdown-toc -->\n* [SSM - Parameter Store](#SSM-ParameterStore)\n\t* [Cache single parameter](#Cachesingleparameter)\n\t* [Change cache expiry](#Changecacheexpiry)\n\t* [Change cache entry settings](#Changecacheentrysettings)\n\t* [Cache multiple parameters](#Cachemultipleparameters)\n\t* [Decorator stacking](#Decoratorstacking)\n\t* [Cache invalidation](#Cacheinvalidation)\n\t* [Return Values](#ReturnValues)\n* [Secrets Manager](#SecretsManager)\n\t* [Cache single secret](#Cachesinglesecret)\n\t* [Change Cache expiry](#ChangeCacheexpiry)\n\t* [Change Cache entry settings](#ChangeCacheentrysettings)\n\t* [Decorator stacking](#Decoratorstacking-1)\n\t* [Cache Invalidation](#CacheInvalidation)\n\t* [Return Values](#ReturnValues-1)\n* [S3](#S3)\n\t* [Cache a single file](#Cacheasinglefile)\n\t* [Change Cache expiry](#ChangeCacheexpiry-1)\n\t* [Check file before download](#Checkfilebeforedownload)\n\n## <a name=\'SSM-ParameterStore\'></a>SSM - Parameter Store\n\n### <a name=\'Cachesingleparameter\'></a>Cache single parameter\n\nTo cache a parameter from ssm, decorate your handler function like so:\n\n```python\nfrom lambda_cache import ssm\n\n@ssm.cache(parameter=\'/production/app/var\')\ndef handler(event, context):\n    var = getattr(context,\'var\')\n    response = do_something(var)\n    return response\n```\nAll invocations of this function over in the next minute will reference the parameter from the function\'s internal cache, rather than making a network call to ssm. After one minute has lapsed, the the next invocation will invoke `get_parameter` to refresh the cache. The parameter value will be injected into the `context` object of your lambda handler.\n\n### <a name=\'Changecacheexpiry\'></a>Change cache expiry\n\nThe default `max_age_in_seconds` settings is 60 seconds (1 minute), it defines the maximum age of a parameter that is acceptable to the handler function. Cache entries older than this, will be refreshed. To set a longer cache duration (e.g 5 minutes), change the setting like so:\n\n```python\nfrom lambda_cache import ssm\n\n@ssm.cache(parameter=\'/production/app/var\', max_age_in_seconds=300)\ndef handler(event, context):\n    var = getattr(context,\'var\')\n    response = do_something(var)\n    return response\n```\n\n_Note: The caching logic runs only at invocation, regardless of how long the function runs. A 15 minute lambda function will not refresh the parameter, unless explicitly refreshed using `get_entry` method. The library is primary interested in caching \'across\' invocation rather than \'within\' an invocation_\n\n### <a name=\'Changecacheentrysettings\'></a>Change cache entry settings\n\nThe default name of the parameter is the string after the last slash(\'/\') character of its name. This means `/production/app/var` and `test/app/var` both resolve to just `var`. To over-ride this default, use `entry_name` setting like so:\n\n```python\nfrom lambda_cache import ssm\n\n@ssm.cache(parameter=\'/production/app/var\', entry_name=\'new_var\')\ndef handler(event, context):\n    var = getattr(context,\'new_var\')\n    response = do_something(var)\n    return response\n```\n\n### <a name=\'Cachemultipleparameters\'></a>Cache multiple parameters\n\nTo cache multiple entries at once, pass a list of parameters to the parameter argument. This method groups all the parameter value under one python dictionary, stored in the Lambda Context under the `entry_name`. \n\n_Note: When using this method, `entry_name` is a required parameter, if not present `NoEntryNameError` exception is thrown._\n\n```python\nfrom lambda_cache import ssm\n\n@ssm.cache(parameter=[\'/app/var1\', \'/app/var2\'], entry_name=\'parameters\')\ndef handler(event, context):\n    var1 = getattr(context,\'parameters\').get(\'var1\')\n    var2 = getattr(context,\'parameters\').get(\'var2\')\n    response = do_something(var)\n    return response\n```\n\nUnder the hood, we use the `get_parameters` API call for boto3, which translate to a single network call for multiple parameters. You can group all parameters types in a single call, including `String`, `StringList` and `SecureString`. `StringList` will return as a list, while all other types will return as plain-text strings. The library does not support returning `SecureString` parameters in encrypted form, and will only return plain-text strings regardless of String type.\n\n_Note: for this method to work, ensure you have both `ssm:GetParameter` **and** `ssm:GetParameters` (with the \'s\' at the end) in your function\'s permission policy_\n\n### <a name=\'Decoratorstacking\'></a>Decorator stacking\n\nIf you wish to cache multiple parameters with different expiry times, stack the decorators. In this example, `var1` will be refreshed every 30 seconds, `var2` will be refreshed after 60.\n\n```python\n@ssm.cache(parameter=\'/production/app/var1\', max_age_in_seconds=30)\n@ssm.cache(parameter=\'/production/app/var2\', max_age_in_seconds=60)\ndef handler(event, context):\n    var1 = getattr(context,\'var1\')\n    var2 = getattr(context,\'var2\')\n    response = do_something(var)\n    return response\n```\n_Note: Decorator stacking performs one API call per decorator, which might result is slower performance_\n\n### <a name=\'Cacheinvalidation\'></a>Cache invalidation\n\nIf you require a fresh value at some point of the code, you can force a refresh using the `ssm.get_entry` function, and setting the `max_age_in_seconds` argument to 0.\n\n```python\nfrom lambda_cache import ssm\n\n@ssm.cache(parameter=\'/prod/var\')\ndef handler(event, context):\n\n    if event.get(\'refresh\'):\n        # refresh parameter\n        var = ssm.get_entry(parameter=\'/prod/var\', max_age_in_seconds=0)\n    else:\n        var = getattr(context,\'var\')\n    \n    response = do_something(var)\n    return response\n```\n\nYou may also use `ssm.get_entry` to get a parameter entry from anywhere in your functions code.\n\nTo only get parameter once in the lifetime of the function, set `max_age_in_seconds` to some arbitary large number ~36000 (10 hours).\n\n### <a name=\'ReturnValues\'></a>Return Values\n\nCaching supports `String`, `SecureString` and `StringList` parameters with no change required (ensure you have `kms:Decrypt` permission for `SecureString`). For simplicity, `StringList` parameters are automatically converted into list (delimited by comma), while `String` and `SecureString` both return the single string value of the parameter.\n\n## <a name=\'SecretsManager\'></a>Secrets Manager\n\n### <a name=\'Cachesinglesecret\'></a>Cache single secret\n\nSecret support is similar, but uses the `secret.cache` decorator.\n\n```python\nfrom lambda_cache import secret\n\n@secret.cache(name=\'/prod/db/conn_string\')\ndef handler(event, context):\n    conn_string = getattr(context,\'conn_string\')\n    return context\n```\n\n\n### <a name=\'ChangeCacheexpiry\'></a>Change Cache expiry\n\nThe default `max_age_in_seconds` settings is 60 seconds (1 minute), it defines how long a parameter should be kept in cache before it is refreshed from ssm. To configure longer or shorter times, modify this argument like so:\n\n```python\nfrom lambda_cache import secret\n\n@secret.cache(name=\'/prod/db/conn_string\', max_age_in_seconds=300)\ndef handler(event, context):\n    var = getattr(context,\'conn_string\')\n    response = do_something(var)\n    return response\n```\n\n_Note: The caching logic runs only at invocation, regardless of how long the function runs. A 15 minute lambda function will not refresh the parameter, unless explicitly refreshed using get_cache_ssm. The library is primary interested in caching \'across\' invocation rather than \'within\' an invocation_\n\n### <a name=\'ChangeCacheentrysettings\'></a>Change Cache entry settings\n\nThe name of the secret is simply shortened to the string after the last slash(\'/\') character of the secret\'s name. This means `/prod/db/conn_string` and `/test/db/conn_string` resolve to just `conn_string`. To over-ride this default, use `entry_name`:\n\n```python\nfrom lambda_cache import secret\n\n@secret.cache(name=\'/prod/db/conn_string\', entry_name=\'new_var\')\ndef handler(event, context):\n    var = getattr(context,\'new_var\')\n    response = do_something(var)\n    return response\n```\n\n### <a name=\'Decoratorstacking-1\'></a>Decorator stacking\n\nIf you wish to cache multiple secrets, you can use decorator stacking.\n\n```python\n@secret.cache(name=\'/prod/db/conn_string\', max_age_in_seconds=30)\n@secret.cache(name=\'/prod/app/elk_username_password\', max_age_in_seconds=60)\ndef handler(event, context):\n    var1 = getattr(context,\'conn_string\')\n    var2 = getattr(context,\'elk_username_password\')\n    response = do_something(var)\n    return response\n```\n\n_Note: Decorator stacking performs one API call per decorator, which might result is slower performance._\n\n### <a name=\'CacheInvalidation\'></a>Cache Invalidation\n\nTo invalidate a secret, use the `get_entry`, setting the `max_age_in_seconds=0`.\n```python\nfrom lambda_cache import secret\n\n@secret.cache(name=\'/prod/db/conn_string\')\ndef handler(event, context):\n\n    if event.get(\'refresh\'):\n        var = secret.get_entry(name=\'/prod/db/conn_string\', max_age_in_seconds=0)\n    else:\n        var = getattr(context,\'conn_string\')\n    response = do_something(var)\n    return response\n```\n\n### <a name=\'ReturnValues-1\'></a>Return Values\n\nSecrets Manager supports both string and binary secrets. For simplicity we will cache the secret in the format it is stored. It is up to the calling application to process the return as Binary or Strings.\n\n## <a name=\'S3\'></a>S3\n\nS3 support is considered _experimental_ for now, but withing the python community we see a lot of folks pull down files from S3 for use in AI/ML models.\n\nFiles downloaded from s3 are automatically stored in the `/tmp` directory of the lambda function. This is the only writable directory within lambda, and has a 512MB of storage space. \n\n### <a name=\'Cacheasinglefile\'></a>Cache a single file\nTo download a file from S3 use the the same decorator pattern:\n\n```python\nfrom lambda_cache import s3\n\n@s3.cache(s3Uri=\'s3://bucket_name/path/to/object.json\')\ndef s3_download_entry_name(event, context):\n\n    # Object from S3 automatically saved to /tmp directory\n    with open("/tmp/object.json") as file_data:\n        status = json.loads(file_data.read())[\'status\']\n\n    return status\n```\n\n### <a name=\'ChangeCacheexpiry-1\'></a>Change Cache expiry\n\nThe default `max_age_in_seconds` settings is 60 seconds (1 minute), it defines how long a file should be kept in `/tmp` before it is refreshed from S3. To configure longer or shorter times, modify this argument like so:\n\n```python\nfrom lambda_cache import s3\n\n@s3.cache(s3Uri=\'s3://bucket_name/path/to/object.json\', max_age_in_seconds=300)\ndef s3_download_entry_name(event, context):\n    with open("/tmp/object.json") as file_data:\n        status = json.loads(file_data.read())[\'status\']\n\n    return status\n```\n\n_Note: The caching logic runs only at invocation, regardless of how long the function runs. A 15 minute lambda function will not refresh the object, unless explicitly refreshed using `s3.get_entry`. The library is primary interested in caching \'across\' invocation rather than \'within\' an invocation_\n\n### <a name=\'Checkfilebeforedownload\'></a>Check file before download\n\nBy default, _lambda_cache_ will download the file once at cache has expired, however, to save on network bandwidth (and possibly time), we can set the `check_before_download` parameter to True. This will check the age of the object in S3 and download **only** if the object has changed since the last download. \n\n```python\nfrom lambda_cache import s3\n\n@s3.cache(s3Uri=\'s3://bucket_name/path/to/object.json\', max_age_in_seconds=300, check_before_download=True)\ndef s3_download_entry_name(event, context):\n    with open("/tmp/object.json") as file_data:\n        status = json.loads(file_data.read())[\'status\']\n\n    return status\n```\n\n_Note: we use the GetHead object call to verify the objects `last_modified_date`. This simplifies the IAM policy of the function, as it still only requires the `s3:GetObject` permission. However, this is still a GET requests, and will be charged as such, for smaller objects it might be cheaper to just download the object_\n\n# Credit\n\nProject inspired by:\n* [SSM-Cache](https://github.com/alexcasalboni/ssm-cache-python)\n* [middy](https://github.com/middyjs/middy)\n',
    'author': 'keithrozario',
    'author_email': 'keith@keithrozario.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/keithrozario/lambda-cache',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)
